﻿/**********************************************************************************************************************
* Gaia Framework for Adobe Flash �2007
* Written by: Steven Sacks
* email: stevensacks@gmail.com
* blog: http://www.stevensacks.net/
* forum: http://www.gaiaflashframework.com/
* By using the Gaia Framework, you agree to keep the above contact information in the source code.

Distributed under Creative Commons Attribution-ShareAlike 3.0 Unported License
http://creativecommons.org/licenses/by-sa/3.0/

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS OF THIS CREATIVE COMMONS PUBLIC LICENSE ("CCPL" OR "LICENSE"). 
THE WORK IS PROTECTED BY COPYRIGHT AND/OR OTHER APPLICABLE LAW. ANY USE OF THE WORK OTHER THAN AS AUTHORIZED UNDER THIS 
LICENSE OR COPYRIGHT LAW IS PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU ACCEPT AND AGREE TO BE BOUND BY THE TERMS OF THIS LICENSE. 
TO THE EXTENT THIS LICENSE MAY BE CONSIDERED TO BE A CONTRACT, THE LICENSOR GRANTS YOU THE RIGHTS CONTAINED HERE IN 
CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND CONDITIONS.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO
THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
**********************************************************************************************************************/

import com.gaiaframework.assets.AbstractAsset;
import com.gaiaframework.assets.AssetProgress;
import mx.utils.Delegate;

class com.gaiaframework.assets.SoundAsset extends AbstractAsset
{	
	private var _clip:MovieClip;
	private var _sound:Sound;
	private var isStreaming:Boolean;
	
	private var _secondOffset:Number;
	private var _loops:Number;
	private var _pos:Number;
	
	// fading and panning
	private var rate:Number = 20;
	private var currentVolume:Number;
	private var currentPan:Number;
	private var fadeInterval:Number;
	private var panInterval:Number;
	private var fadeCallback:Function;
	private var panCallback:Function;
	
	// streaming loops
	private var streamLoops:Number;
	private var loopStreamDelegate:Function;
	
	function SoundAsset()
	{
		super();
		_pos = 0;
		_loops = 0;
		streamLoops = 0;
		_secondOffset = 0;
		currentVolume = 100;
		currentPan = 0;
		loopStreamDelegate = Delegate.create(this, loopStreamingSound);
	}
	public function get sound():Sound
	{
		return _sound;
	}
	public function get streaming():Boolean
	{
		return isStreaming;
	}
	public function set streaming(b:Boolean):Void
	{
		isStreaming = b;
	}
	public function init(target:MovieClip):Void
	{
		isActive = true;
		_clip = target;
		_sound = new Sound(_clip);
	}
	public function preload():Void
	{
		_sound.loadSound(src, isStreaming);
		if (isStreaming) _sound.stop();
		super.load();
	}
	public function load(secondOffset:Number, loops:Number):Void
	{
		_sound.stop();
		_loops = streamLoops = loops || 0;
		if (isStreaming)
		{
			setStreamingLoop();
			_sound.loadSound(src, isStreaming);
		}
		else
		{
			if (percentLoaded == 1)
			{
				start(secondOffset, loops);
			}
			else
			{
				// load event sound and play after
				_sound.onLoad = Delegate.create(this, onEventSoundLoaded);
				_secondOffset = secondOffset;
				if (showProgress) 
				{
					AssetProgress.instance.load(this);
					super.load();
				}
				_sound.loadSound(src);
			}
		}
		_sound.setVolume(currentVolume);
		_sound.setPan(currentPan);
	}
	public function getBytesLoaded():Number
	{
		return _sound.getBytesLoaded();
	}
	public function getBytesTotal():Number
	{
		return _sound.getBytesTotal();
	}
	public function destroy():Void
	{
		delete _sound;
		_clip.removeMovieClip();
		_clip = null;
		super.destroy();
	}
	public function retry():Void
	{
		_sound.loadSound(src, isStreaming);
		_sound.stop();
	}
	
	// PUBLIC UTILITY FUNCTIONS
	public function pause(flag:Boolean):Void
	{
		if (flag)
		{
			_pos = sound.position / 1000;
			sound.stop();
		}
		else
		{
			sound.start(_pos, _loops);
		}
	}
	public function fadeTo(value:Number, duration:Number, callback:Function):Void
	{
		clearInterval(fadeInterval);
		if (duration > 0)
		{
			var initValue:Number = sound.getVolume();
			var delta:Number = Math.round(value) - initValue;
			var initTime:Number = getTimer();
			if (delta != 0)
			{
				fadeCallback = callback;
				fadeInterval = setInterval(this, "fadeStep", rate, initTime, initValue, delta, duration);
			}
			else
			{
				callback();
			}
		}
		else
		{
			callback();
		}
	}
	public function panTo(value:Number, duration:Number, callback:Function):Void
	{
		clearInterval(panInterval);
		if (duration > 0)
		{
			var initValue:Number = sound.getPan();
			var delta:Number = Math.round(value) - initValue;
			var initTime:Number = getTimer();
			if (delta != 0)
			{
				panCallback = callback;
				panInterval = setInterval(this, "panStep", rate, initTime, initValue, delta, duration);
			}
			else
			{
				callback();
			}
		}
		else
		{
			callback();
		}
	}
	// PRIVATE UTILITY FUNCTIONS
	private function fadeStep(initTime:Number, initValue:Number, delta:Number, duration:Number):Void
	{
		var endTime:Number = getTimer() - initTime;
		var value:Number = easingEquation(endTime, initValue, delta, duration);
		setVolume(value);
		if(endTime >= duration)
		{
			clearInterval(fadeInterval);
			fadeCallback();
		}
	}
	private function panStep(initTime:Number, initValue:Number, delta:Number, duration:Number):Void
	{
		var endTime:Number = getTimer() - initTime;
		var value:Number = easingEquation(endTime, initValue, delta, duration);
		setPan(value);
		if(endTime >= duration)
		{
			clearInterval(panInterval);
			panCallback();
		}
	}
	private function onEventSoundLoaded():Void
	{
		sound.start(_secondOffset, _loops);
		_secondOffset = 0;
	}
	private function setStreamingLoop():Void
	{
		if (streamLoops > 0) _sound.onSoundComplete = loopStreamDelegate;
		else if (_sound.onSoundComplete == loopStreamDelegate) delete _sound.onSoundComplete;
	}
	private function loopStreamingSound():Void
	{
		_sound.start(0, --streamLoops);
	}
	// linear easing courtesy Robert Penner
	private function easingEquation(t:Number, b:Number, c:Number, d:Number):Number
	{
		return c / 2 * (Math.sin(Math.PI * (t / d - 0.5)) + 1) + b;
	}
	// used by SoundClipAsset for MTASC compliance (super.super not allowed)
	public function get abstract():AbstractAsset
	{
		return super;
	}
	
	// SOUND GROUP PRELOAD
	public function soundGroupPreload(secondOffset:Number, loops:Number):Void
	{
		_sound.stop();
		_secondOffset = secondOffset || 0;
		_loops = streamLoops = loops || 0;
		if (percentLoaded != 1 && showProgress) 
		{
			AssetProgress.instance.load(this);
			super.load();
		}
		if (isStreaming) setStreamingLoop();
		_sound.loadSound(src, isStreaming);
		_sound.setVolume(currentVolume);
		_sound.setPan(currentPan);
		_sound.stop();
	}
	
	// PROXY SOUND PROPS/FUNCS
	public function loadSound(url:String, stream:Boolean):Void
	{
		isStreaming = stream;
		_sound.loadSound(url, stream);
		_sound.setVolume(currentVolume);
		_sound.setPan(currentPan);
	}
	public function start(secondOffset:Number, loops:Number):Void
	{
		_loops = streamLoops = loops;
		if (isStreaming) setStreamingLoop();
		sound.start(secondOffset, loops);
	}
	public function stop():Void
	{
		sound.stop();
	}
	public function getVolume():Number
	{
		return sound.getVolume() || currentVolume;
	}
	public function setVolume(value:Number):Void
	{
		currentVolume = value;
		sound.setVolume(currentVolume);
	}
	public function get duration():Number
	{
		return sound.duration;
	}
	public function getPan():Number
	{
		return sound.getPan() || currentPan;
	}
	public function setPan(value:Number):Void
	{
		currentPan = value;
		sound.setPan(currentPan);
	}
	public function get position():Number
	{
		return sound.position;
	}
	public function getTransform():Object
	{
		return sound.getTransform();
	}
	public function setTransform(transformObject:Object):Void
	{
		sound.setTransform(transformObject);
	}
	public function get id3():Object
	{
		return sound.id3;
	}	
	public function set onSoundComplete(func:Function):Void
	{
		sound.onSoundComplete = func;
	}
	public function set onLoad(func:Function):Void
	{
		_sound.onLoad = func;
	}
}