﻿/**********************************************************************************************************************
* Gaia Framework for Adobe Flash �2007
* Written by: Steven Sacks
* email: stevensacks@gmail.com
* blog: http://www.stevensacks.net/
* forum: http://www.gaiaflashframework.com/
* By using the Gaia Framework, you agree to keep the above contact information in the source code.

Distributed under Creative Commons Attribution-ShareAlike 3.0 Unported License
http://creativecommons.org/licenses/by-sa/3.0/

THE WORK (AS DEFINED BELOW) IS PROVIDED UNDER THE TERMS OF THIS CREATIVE COMMONS PUBLIC LICENSE ("CCPL" OR "LICENSE"). 
THE WORK IS PROTECTED BY COPYRIGHT AND/OR OTHER APPLICABLE LAW. ANY USE OF THE WORK OTHER THAN AS AUTHORIZED UNDER THIS 
LICENSE OR COPYRIGHT LAW IS PROHIBITED.

BY EXERCISING ANY RIGHTS TO THE WORK PROVIDED HERE, YOU ACCEPT AND AGREE TO BE BOUND BY THE TERMS OF THIS LICENSE. 
TO THE EXTENT THIS LICENSE MAY BE CONSIDERED TO BE A CONTRACT, THE LICENSOR GRANTS YOU THE RIGHTS CONTAINED HERE IN 
CONSIDERATION OF YOUR ACCEPTANCE OF SUCH TERMS AND CONDITIONS.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO
THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
**********************************************************************************************************************/

import net.stevensacks.utils.ObservableClass;
import com.gaiaframework.assets.SoundClipAsset;
import com.gaiaframework.assets.SoundAsset;
import mx.utils.Delegate;

class com.gaiaframework.utils.SoundGroup extends ObservableClass
{
	private var assets:Array;
	private var volumes:Array;
	private var pans:Array;
	
	private var volume:Number;
	private var pan:Number;	
	private var transform:Object;
	
	private var fadeCallbackCount:Number;
	private var panCallbackCount:Number;
	private var fadeCallbackFunction:Function;
	private var panCallbackFunction:Function;
	
	private var preloadInterval:Number;
	
	private var index:Number;

	// Pass as many assets as you want in the group
	function SoundGroup()
	{
		super();
		assets = [];
		volumes = [];
		pans = [];
		index = -1;
		volume = 1;
		pan = 0;
		for (var i:Number = 0; i < arguments.length; i++)
		{
			if (arguments[i] instanceof SoundAsset || arguments[i] instanceof SoundClipAsset)
			{
				assets.push(arguments[i]);
				volumes.push(arguments[i].getVolume());
				pans.push(arguments[i].getPan());
			}
			else
			{
				for (var a:String in arguments[i])
				{
					if (arguments[i][a] instanceof SoundAsset || arguments[i][a] instanceof SoundClipAsset)
					{
						assets.push(arguments[i][a]);
						volumes.push(arguments[i][a].getVolume());
						pans.push(arguments[i][a].getPan());
					}
				}
			}
		}
	}
	
	// PUBLIC METHODS
	public function addSound(sound:SoundAsset):Void
	{
		var i:Number = assets.length;
		var match:Boolean = false;
		while (i--)
		{
			if (assets[i] == sound)
			{
				match = true;
				break;
			}
		}
		if (!match) 
		{
			assets.push(sound);
			volumes.push(sound.getVolume());
			pans.push(sound.getPan());
		}
	}
	public function removeSound(sound:SoundAsset):Void
	{
		var i:Number = assets.length;
		while (i--)
		{
			if (assets[i] == sound)
			{
				assets.splice(i, 1);
				volumes.splice(i, 1);
				pans.splice(i, 1);
				break;
			}
		}
	}
	
	// Calibrate resets the group volume to 1, current pan to 0, and sets volumes and pans to the current sound asset and pans
	public function calibrate():Void
	{
		var i:Number = assets.length;
		volume = 1;
		pan = 0;
		volumes = [];
		pans = [];
		while (i--)
		{
			volumes[i] = assets[i].getVolume();
			pans[i] = assets[i].getPan();
		}
	}
	
	// Load all sounds in group for simultaneous or instant playback
	public function preload(playWhenLoaded:Boolean, secondOffset:Number, loops:Number):Void
	{
		var i:Number = assets.length;
		var loadedCount:Number = 0;
		while (i--)
		{
			var sound:SoundAsset = assets[i];
			if (sound.percentLoaded == 1 && sound.sound != undefined)
			{
				loadedCount++;
			}
			else
			{
				sound.soundGroupPreload(secondOffset, loops);
			}
		}
		if (loadedCount < assets.length)
		{
			clearInterval(preloadInterval);
			preloadInterval = setInterval(this, "checkAllSoundsLoaded", 30, playWhenLoaded, secondOffset, loops);
		}
		else
		{
			if (playWhenLoaded) playAll(secondOffset, loops);
			dispatchEvent({type:"preloaded"});
		}
	}
	
	// Pick a sound at random and play it
	public function playRandom(secondOffset:Number, loops:Number, repeat:Boolean):SoundAsset
	{
		var sound:SoundAsset = assets[index = getRandomIndex(repeat)];
		sound.load(secondOffset, loops);
		return sound;
	}
	// Play next sound in the array - loop back to beginning if at the end
	public function playNext(secondOffset:Number, loops:Number):SoundAsset
	{
		if (++index >= assets.length) index = 0;
		var sound:SoundAsset = assets[index];
		sound.load(secondOffset, loops);
		return sound;
	}
	// Play previous sound in the array - loop back to end if at the beginning
	public function playPrevious(secondOffset:Number, loops:Number):SoundAsset
	{
		if (--index < 0) index = assets.length - 1;
		var sound:SoundAsset = assets[index];
		sound.load(secondOffset, loops);
		return sound;
	}
		
	// Play all sounds in the group
	public function play(secondOffset:Number, loops:Number):Void
	{
		getAllSoundsLoaded() ? playAll(secondOffset, loops) : preload(true, secondOffset, loops);
	}
	
	// Stop all sounds in the group
	public function stop():Void
	{
		var i:Number = assets.length;
		while (i--)
		{
			var sound:SoundAsset = assets[i];
			sound.stop();
		}
	}
	
	// Pause all sounds in the group
	public function pause(flag:Boolean):Void
	{
		var i:Number = assets.length;
		while (i--)
		{
			var sound:SoundAsset = assets[i];
			sound.pause(flag);
		}
	}
	
	// Fade all sounds in the group
	public function fadeTo(value:Number, duration:Number, callback:Function):Void
	{
		fadeCallbackCount = assets.length;
		fadeCallbackFunction = callback;
		volume = Math.round(value) / 100;
		var i:Number = assets.length;
		while (i--)
		{
			var sound:SoundAsset = assets[i];
			var adjVolume:Number = Math.round(volumes[i] * volume);
			sound.fadeTo(adjVolume, duration, Delegate.create(this, fadeCallback));
		}
	}
	private function fadeCallback():Void
	{
		if (--fadeCallbackCount == 0) fadeCallbackFunction();
	}
	
	// Pan all sounds in the group
	public function panTo(value:Number, duration:Number, callback:Function):Void
	{
		panCallbackCount = assets.length;
		panCallbackFunction = callback;
		pan = Math.round(Math.min(100, Math.max(-100, value)));
		var i:Number = assets.length;
		while (i--)
		{
			var sound:SoundAsset = assets[i];
			var adjPan:Number = Math.min(100, Math.max(-100, pans[i] + pan));
			sound.panTo(adjPan, duration, Delegate.create(this, panCallback));
		}
	}
	private function panCallback():Void
	{
		if (--panCallbackCount == 0) panCallbackFunction();
	}
	
	// Adjusts Volume as a percentage of individual asset volumes
	public function setVolume(value:Number):Void
	{
		volume = value / 100;
		var i:Number = assets.length;
		while (i--)
		{
			var sound:SoundAsset = assets[i];
			var newVolume:Number = Math.round(volumes[i] * volume);
			sound.setVolume(newVolume);
		}
	}
	public function getVolume():Number
	{
		return volume * 100;
	}
	
	// Adjusts Pan as an offset of individual asset pans
	public function setPan(value:Number):Void
	{
		pan = value;
		var i:Number = assets.length;
		while (i--)
		{
			var sound:SoundAsset = assets[i];
			var newPan:Number = Math.min(100, Math.max(-100, pans[i] + pan));
			sound.setPan(newPan);
		}
	}
	public function getPan():Number
	{
		return pan;
	}
	
	// Global Transform for all sounds in the group - Must be set before you can get
	public function setTransform(transformObject:Object):Void
	{
		transform = transformObject;
		var i:Number = assets.length;
		while (i--)
		{
			var sound:SoundAsset = assets[i];
			sound.setTransform(transform);
		}
	}
	public function getTransform():Object
	{
		return transform;
	}
	
	// PRIVATE UTILITIES
	private function getRandomIndex(repeat:Boolean):Number
	{
		var rnd:Number = Math.round(Math.random() * (assets.length - 1));
		if (repeat || rnd != index || assets.length == 1) return rnd;
		return getRandomIndex();
	}
	private function checkAllSoundsLoaded(playWhenLoaded:Boolean, secondOffset:Number, loops:Number):Void
	{
		if (getAllSoundsLoaded())
		{
			clearInterval(preloadInterval);
			if (playWhenLoaded) playAll(secondOffset, loops);
			dispatchEvent({type:"preloaded"});
		}
	}
	private function getAllSoundsLoaded():Boolean
	{
		var i:Number = assets.length;
		while (i--)
		{
			var sound:SoundAsset = assets[i];
			if (sound.percentLoaded != 1) return false;
		}
		return true;
	}
	private function playAll(secondOffset:Number, loops:Number):Void
	{
		var i:Number = assets.length;
		while (i--)
		{
			var sound:SoundAsset = assets[i];
			sound.start(secondOffset, loops);
		}
	}
}